(** [Lucid] is a super simple logging library. *)

(** {1 Log Levels} *)

type level = Output | Debug | Info | Warn | Error | Fatal

val level_to_string : level option -> string
(** Converts Level to String *)

val level_of_string : string -> (level option, string) result
(** Converts String to Level *)

val level_to_int : level option -> int
(** Converts Level to Int
    {[
        Debug -> 1
        Info -> 2
        Warn -> 3
        Error -> 4
        Fatal -> 5
    ]}
 *)

val level_of_int : int -> (level option, string) result
(** Converts Int to level *)

val set_level : level -> unit
(** [set_level Warn] will set enable logging for [Warn] and the levels above it. *)

val get_level : unit -> level
(** Returns the lowest level being logged. *)


(** {1 Timestamp} *)

type timezone = Local | GMT | None

val set_tz : timezone -> unit
(** [set_tz GMT] will set the timestamp timezone to GMT. *)

val get_tz : unit -> timezone
(** Returns the timezone being used for the timestamp. *)

val disable_tz : unit -> unit
(** Disables timestamps, timestamps can be enabled again using [set_tz]. *)

(** {1 Log Functions} *)

val debug : string -> unit
(** [debug "message"] will log [message] with the level [Debug]. *)

val info : string -> unit
(** [info "message"] will log [message] with the level [Info]. *)

val warn : string -> unit
(** [warn "message"] will log [message] with the level [Warn]. *)

val error : string -> unit
(** [error "message"] will log [message] with the level [Error]. *)

val fatal : string -> unit
(** [fatal "message"] will log [message] with the level [Fatal]. *)

val critical : string -> unit
(** Same as [fatal] 

    [critical "message"] will log [message] with the level [Fatal].
 *)
 

(** {1 Output Channel} *)

val set_out_channel : out_channel -> unit
val get_out_channel : unit -> out_channel


(** {1 Colors} *)

type color = Black | Red | Green | Yellow | Blue | Magenta | Cyan | White | Custom of int

val set_color_fg : clr:color -> lvl:level -> unit
(** Set foreground color of the logging label for a certain level. *)

val set_color_fg_brt : clr:color -> lvl:level -> unit
(** Same as [set_color_fg] but uses bright colors. *)

val set_color_bg : clr:color -> lvl:level -> unit
(** Set background color of the logging label for a certain level. *)

val set_color_bg_brt : clr:color -> lvl:level -> unit
(** Same as [set_color_bg] but uses bright colors. *)

val get_color : level -> string * string
(** [get_color Error] will return a 2-tuple [string * string] where the first string corresponds to the foreground color of the level [Error] and second to the background color. *)


(** {1 Counters} *)

val get_count : level -> int
(** Gets the number of times a certain level has been successfully called to log. *)

val get_count_logged : level -> int
(** Gets the number of times a certain level has actually logged. *)

val get_count_unlogged : level -> int
(** [get_count] - [get_count_logged] *)

val set_count : lvl:level -> int -> unit

val set_count_logged : lvl:level -> int -> unit

val count_total : unit -> int

val count_total_logged : unit -> int